// Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved

package hints

import (
	"encoding/json"
	"strings"
	"testing"
	"time"

	"gitlab.bit9.local/psc/ems/hinting/internal/utils"
	"gitlab.bit9.local/psc/ems/hinting/pkg/hints/hinttype"
)

var (
	lut  = time.UnixMilli(1677888000000) // Monday, January 2, 2023 0:00:00
	llqt = time.UnixMilli(1672617600000) // Saturday, March 4, 2023 0:00:00
)

////////////////////////////////////////////////////////////////////////////////
// These tests validate HintResult marshaling
////////////////////////////////////////////////////////////////////////////////

// Ensure that a filled out HintResult can be properly marshaled
func Test_HintResult(t *testing.T) {
	t.Logf(utils.HeaderStr("Full HintResult Marshaled"))
	hr := HintResult{
		LastLiveQueryTime: &llqt, LastUpdateTime: &lut,
		Hints: hinttype.NewHintSet(hinttype.NEW_POLICY, hinttype.NEW_CONFIG_DATA),
	}
	t.Logf("Using HintResult: %+v, Hints: %+v", hr, *hr.Hints) // Hints should have a value of 5
	bytes, err := json.Marshal(&hr)
	if err != nil {
		t.Fatalf("Failed to unmarshal json: %v", err)
	}
	conv := string(bytes)
	if !strings.Contains(conv, "NEW_POLICY") || !strings.Contains(conv, "NEW_CONFIG_DATA") {
		t.Fatalf("Failed to marshal hints")
	}
	if !strings.Contains(conv, `"last_live_query_time":1672617600000`) {
		t.Fatalf("Failed to marshal last_live_query_time")
	}
	if !strings.Contains(conv, `"last_update_time":1677888000000`) {
		t.Fatalf("Failed to marshal last_update_time")
	}
}

// Ensure that a HintResult with no hints can be properly marshaled
func Test_HintResultNoHints(t *testing.T) {
	t.Logf(utils.HeaderStr("HintResult Marshaled, no Hints"))
	hr := HintResult{LastLiveQueryTime: &NOW, LastUpdateTime: &NOW, Hints: hinttype.NewHintSet()}
	t.Logf("Using Hints: %+v", *hr.Hints) // Hints should have a value of 0
	bytes, err := json.Marshal(&hr)
	if err != nil {
		t.Fatalf("Failed to unmarshal json: %v", err)
	}

	if !strings.Contains(string(bytes), `"hints":[]`) {
		t.Fatalf("Failed to marshal correctly with empty hints")
	}
}

// Ensure that a hint result with missing hints is guard-railed to empty hints
func Test_HintResultMissingHints(t *testing.T) {
	t.Logf(utils.HeaderStr("HintResult Marshaled, missing Hints"))
	hr := HintResult{LastLiveQueryTime: &llqt, LastUpdateTime: &lut}
	t.Logf("Using HintResult: %+v", hr)
	bytes, err := json.Marshal(&hr)
	if err != nil {
		t.Fatalf("Failed to unmarshal json: %v", err)
	}

	if !strings.Contains(string(bytes), `"hints":[]`) {
		t.Fatalf("Failed to marshal correctly with no hints: %s", string(bytes))
	}
}

// Ensure that a hint result with missing timestamps is allowed
func Test_HintResultMissingTimes(t *testing.T) {
	t.Logf(utils.HeaderStr("HintResult Marshaled, missing timestamps"))
	hr := HintResult{Hints: hinttype.NewHintSet()}
	t.Logf("Using HintResult: %+v", hr)
	bytes, err := json.Marshal(&hr)
	if err != nil {
		t.Fatalf("Failed to unmarshal json: %v", err)
	}

	conv := string(bytes)
	if !strings.Contains(conv, `"hints":[]`) {
		t.Fatalf("Failed to marshal correctly with no hints")
	}
	if strings.Contains(conv, "last_update_time") || strings.Contains(conv, "last_live_query_time") {
		t.Fatalf("Incorrectly added timestamps")
	}
}
